/*
 * Copyright (C) 2007-2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#ifdef __OBJC__
#error Please use @autoreleasepool instead of AutodrainedPool.
#endif

#include <wtf/Noncopyable.h>

namespace WTF {

// This class allows non-Objective-C C++ code to create an autorelease pool.
// It cannot be used in Objective-C++ code, won't be compiled; instead @autoreleasepool should be used.
// It can be used in cross-platform code; will compile down to nothing for non-Cocoa platforms.

class AutodrainedPool {
    WTF_MAKE_NONCOPYABLE(AutodrainedPool);

public:
#if USE(FOUNDATION)
    WTF_EXPORT_PRIVATE AutodrainedPool();
    WTF_EXPORT_PRIVATE ~AutodrainedPool();
#else
    AutodrainedPool() { }
    ~AutodrainedPool() { }
#endif

private:
#if USE(FOUNDATION)
    void* m_autoreleasePool;
#endif
};

} // namespace WTF

using WTF::AutodrainedPool;
