/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 * Copyright (C) 2013-2016 Apple Inc. All rights reserved.
 * Copyright (C) 2014 University of Washington. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

// DO NOT EDIT THIS FILE. It is automatically generated from Automation.json
// by the script: Source/JavaScriptCore/inspector/scripts/generate-inspector-protocol-bindings.py

#include "config.h"
#include "AutomationBackendDispatchers.h"

#include <JavaScriptCore/InspectorFrontendRouter.h>
#include <wtf/JSONValues.h>
#include <wtf/NeverDestroyed.h>
#include <wtf/text/CString.h>

namespace Inspector {

AutomationBackendDispatcherHandler::~AutomationBackendDispatcherHandler() { }

Ref<AutomationBackendDispatcher> AutomationBackendDispatcher::create(BackendDispatcher& backendDispatcher, AutomationBackendDispatcherHandler* agent)
{
    return adoptRef(*new AutomationBackendDispatcher(backendDispatcher, agent));
}

AutomationBackendDispatcher::AutomationBackendDispatcher(BackendDispatcher& backendDispatcher, AutomationBackendDispatcherHandler* agent)
    : SupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain("Automation"_s, this);
}

void AutomationBackendDispatcher::dispatch(long requestId, const String& method, Ref<JSON::Object>&& message)
{
    Ref<AutomationBackendDispatcher> protect(*this);

    RefPtr<JSON::Object> parameters;
    message->getObject("params"_s, parameters);

    typedef void (AutomationBackendDispatcher::*CallHandler)(long requestId, RefPtr<JSON::Object>&& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    static NeverDestroyed<DispatchMap> dispatchMap;
    if (dispatchMap.get().isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "getBrowsingContexts", &AutomationBackendDispatcher::getBrowsingContexts },
            { "getBrowsingContext", &AutomationBackendDispatcher::getBrowsingContext },
            { "createBrowsingContext", &AutomationBackendDispatcher::createBrowsingContext },
            { "closeBrowsingContext", &AutomationBackendDispatcher::closeBrowsingContext },
            { "switchToBrowsingContext", &AutomationBackendDispatcher::switchToBrowsingContext },
            { "setWindowFrameOfBrowsingContext", &AutomationBackendDispatcher::setWindowFrameOfBrowsingContext },
            { "maximizeWindowOfBrowsingContext", &AutomationBackendDispatcher::maximizeWindowOfBrowsingContext },
            { "hideWindowOfBrowsingContext", &AutomationBackendDispatcher::hideWindowOfBrowsingContext },
            { "navigateBrowsingContext", &AutomationBackendDispatcher::navigateBrowsingContext },
            { "goBackInBrowsingContext", &AutomationBackendDispatcher::goBackInBrowsingContext },
            { "goForwardInBrowsingContext", &AutomationBackendDispatcher::goForwardInBrowsingContext },
            { "reloadBrowsingContext", &AutomationBackendDispatcher::reloadBrowsingContext },
            { "waitForNavigationToComplete", &AutomationBackendDispatcher::waitForNavigationToComplete },
            { "evaluateJavaScriptFunction", &AutomationBackendDispatcher::evaluateJavaScriptFunction },
            { "performMouseInteraction", &AutomationBackendDispatcher::performMouseInteraction },
            { "performKeyboardInteractions", &AutomationBackendDispatcher::performKeyboardInteractions },
            { "performInteractionSequence", &AutomationBackendDispatcher::performInteractionSequence },
            { "cancelInteractionSequence", &AutomationBackendDispatcher::cancelInteractionSequence },
            { "takeScreenshot", &AutomationBackendDispatcher::takeScreenshot },
            { "resolveChildFrameHandle", &AutomationBackendDispatcher::resolveChildFrameHandle },
            { "resolveParentFrameHandle", &AutomationBackendDispatcher::resolveParentFrameHandle },
            { "computeElementLayout", &AutomationBackendDispatcher::computeElementLayout },
            { "selectOptionElement", &AutomationBackendDispatcher::selectOptionElement },
            { "isShowingJavaScriptDialog", &AutomationBackendDispatcher::isShowingJavaScriptDialog },
            { "dismissCurrentJavaScriptDialog", &AutomationBackendDispatcher::dismissCurrentJavaScriptDialog },
            { "acceptCurrentJavaScriptDialog", &AutomationBackendDispatcher::acceptCurrentJavaScriptDialog },
            { "messageOfCurrentJavaScriptDialog", &AutomationBackendDispatcher::messageOfCurrentJavaScriptDialog },
            { "setUserInputForCurrentJavaScriptPrompt", &AutomationBackendDispatcher::setUserInputForCurrentJavaScriptPrompt },
            { "setFilesToSelectForFileUpload", &AutomationBackendDispatcher::setFilesToSelectForFileUpload },
            { "getAllCookies", &AutomationBackendDispatcher::getAllCookies },
            { "deleteSingleCookie", &AutomationBackendDispatcher::deleteSingleCookie },
            { "addSingleCookie", &AutomationBackendDispatcher::addSingleCookie },
            { "deleteAllCookies", &AutomationBackendDispatcher::deleteAllCookies },
            { "getSessionPermissions", &AutomationBackendDispatcher::getSessionPermissions },
            { "setSessionPermissions", &AutomationBackendDispatcher::setSessionPermissions },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.get().add(commands[i].name, commands[i].handler);
    }

    auto findResult = dispatchMap.get().find(method);
    if (findResult == dispatchMap.get().end()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::MethodNotFound, "'Automation." + method + "' was not found");
        return;
    }

    ((*this).*findResult->value)(requestId, WTFMove(parameters));
}

AutomationBackendDispatcherHandler::GetBrowsingContextsCallback::GetBrowsingContextsCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::GetBrowsingContextsCallback::sendSuccess(RefPtr<JSON::ArrayOf<Inspector::Protocol::Automation::BrowsingContext>>&& contexts)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setArray("contexts"_s, contexts);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::getBrowsingContexts(long requestId, RefPtr<JSON::Object>&&)
{
    Ref<AutomationBackendDispatcherHandler::GetBrowsingContextsCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::GetBrowsingContextsCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->getBrowsingContexts(callback.copyRef());

}

AutomationBackendDispatcherHandler::GetBrowsingContextCallback::GetBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::GetBrowsingContextCallback::sendSuccess(RefPtr<Inspector::Protocol::Automation::BrowsingContext>&& context)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setObject("context"_s, context);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::getBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.getBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::GetBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::GetBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->getBrowsingContext(in_handle, callback.copyRef());

}

AutomationBackendDispatcherHandler::CreateBrowsingContextCallback::CreateBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::CreateBrowsingContextCallback::sendSuccess(const String& handle, Inspector::Protocol::Automation::BrowsingContextPresentation presentation)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setString("handle"_s, handle);
    jsonMessage->setString("presentation"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(presentation));
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::createBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    bool opt_in_presentationHint_valueFound = false;
    String opt_in_presentationHint = m_backendDispatcher->getString(parameters.get(), "presentationHint"_s, &opt_in_presentationHint_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.createBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::CreateBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::CreateBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->createBrowsingContext(opt_in_presentationHint_valueFound ? &opt_in_presentationHint : nullptr, callback.copyRef());

}

void AutomationBackendDispatcher::closeBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.closeBrowsingContext' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->closeBrowsingContext(error, in_handle);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

AutomationBackendDispatcherHandler::SwitchToBrowsingContextCallback::SwitchToBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::SwitchToBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::switchToBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.switchToBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::SwitchToBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::SwitchToBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->switchToBrowsingContext(in_browsingContextHandle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::SetWindowFrameOfBrowsingContextCallback::SetWindowFrameOfBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::SetWindowFrameOfBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::setWindowFrameOfBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_origin_valueFound = false;
    RefPtr<JSON::Object> opt_in_origin = m_backendDispatcher->getObject(parameters.get(), "origin"_s, &opt_in_origin_valueFound);
    bool opt_in_size_valueFound = false;
    RefPtr<JSON::Object> opt_in_size = m_backendDispatcher->getObject(parameters.get(), "size"_s, &opt_in_size_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.setWindowFrameOfBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::SetWindowFrameOfBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::SetWindowFrameOfBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->setWindowFrameOfBrowsingContext(in_handle, opt_in_origin_valueFound ? opt_in_origin.get() : nullptr, opt_in_size_valueFound ? opt_in_size.get() : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::MaximizeWindowOfBrowsingContextCallback::MaximizeWindowOfBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::MaximizeWindowOfBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::maximizeWindowOfBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.maximizeWindowOfBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::MaximizeWindowOfBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::MaximizeWindowOfBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->maximizeWindowOfBrowsingContext(in_handle, callback.copyRef());

}

AutomationBackendDispatcherHandler::HideWindowOfBrowsingContextCallback::HideWindowOfBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::HideWindowOfBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::hideWindowOfBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.hideWindowOfBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::HideWindowOfBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::HideWindowOfBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->hideWindowOfBrowsingContext(in_handle, callback.copyRef());

}

AutomationBackendDispatcherHandler::NavigateBrowsingContextCallback::NavigateBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::NavigateBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::navigateBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    String in_url = m_backendDispatcher->getString(parameters.get(), "url"_s, nullptr);
    bool opt_in_pageLoadStrategy_valueFound = false;
    String opt_in_pageLoadStrategy = m_backendDispatcher->getString(parameters.get(), "pageLoadStrategy"_s, &opt_in_pageLoadStrategy_valueFound);
    bool opt_in_pageLoadTimeout_valueFound = false;
    int opt_in_pageLoadTimeout = m_backendDispatcher->getInteger(parameters.get(), "pageLoadTimeout"_s, &opt_in_pageLoadTimeout_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.navigateBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::NavigateBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::NavigateBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->navigateBrowsingContext(in_handle, in_url, opt_in_pageLoadStrategy_valueFound ? &opt_in_pageLoadStrategy : nullptr, opt_in_pageLoadTimeout_valueFound ? &opt_in_pageLoadTimeout : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::GoBackInBrowsingContextCallback::GoBackInBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::GoBackInBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::goBackInBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_pageLoadStrategy_valueFound = false;
    String opt_in_pageLoadStrategy = m_backendDispatcher->getString(parameters.get(), "pageLoadStrategy"_s, &opt_in_pageLoadStrategy_valueFound);
    bool opt_in_pageLoadTimeout_valueFound = false;
    int opt_in_pageLoadTimeout = m_backendDispatcher->getInteger(parameters.get(), "pageLoadTimeout"_s, &opt_in_pageLoadTimeout_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.goBackInBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::GoBackInBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::GoBackInBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->goBackInBrowsingContext(in_handle, opt_in_pageLoadStrategy_valueFound ? &opt_in_pageLoadStrategy : nullptr, opt_in_pageLoadTimeout_valueFound ? &opt_in_pageLoadTimeout : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::GoForwardInBrowsingContextCallback::GoForwardInBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::GoForwardInBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::goForwardInBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_pageLoadStrategy_valueFound = false;
    String opt_in_pageLoadStrategy = m_backendDispatcher->getString(parameters.get(), "pageLoadStrategy"_s, &opt_in_pageLoadStrategy_valueFound);
    bool opt_in_pageLoadTimeout_valueFound = false;
    int opt_in_pageLoadTimeout = m_backendDispatcher->getInteger(parameters.get(), "pageLoadTimeout"_s, &opt_in_pageLoadTimeout_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.goForwardInBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::GoForwardInBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::GoForwardInBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->goForwardInBrowsingContext(in_handle, opt_in_pageLoadStrategy_valueFound ? &opt_in_pageLoadStrategy : nullptr, opt_in_pageLoadTimeout_valueFound ? &opt_in_pageLoadTimeout : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::ReloadBrowsingContextCallback::ReloadBrowsingContextCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::ReloadBrowsingContextCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::reloadBrowsingContext(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_pageLoadStrategy_valueFound = false;
    String opt_in_pageLoadStrategy = m_backendDispatcher->getString(parameters.get(), "pageLoadStrategy"_s, &opt_in_pageLoadStrategy_valueFound);
    bool opt_in_pageLoadTimeout_valueFound = false;
    int opt_in_pageLoadTimeout = m_backendDispatcher->getInteger(parameters.get(), "pageLoadTimeout"_s, &opt_in_pageLoadTimeout_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.reloadBrowsingContext' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::ReloadBrowsingContextCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::ReloadBrowsingContextCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->reloadBrowsingContext(in_handle, opt_in_pageLoadStrategy_valueFound ? &opt_in_pageLoadStrategy : nullptr, opt_in_pageLoadTimeout_valueFound ? &opt_in_pageLoadTimeout : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::WaitForNavigationToCompleteCallback::WaitForNavigationToCompleteCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::WaitForNavigationToCompleteCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::waitForNavigationToComplete(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    bool opt_in_pageLoadStrategy_valueFound = false;
    String opt_in_pageLoadStrategy = m_backendDispatcher->getString(parameters.get(), "pageLoadStrategy"_s, &opt_in_pageLoadStrategy_valueFound);
    bool opt_in_pageLoadTimeout_valueFound = false;
    int opt_in_pageLoadTimeout = m_backendDispatcher->getInteger(parameters.get(), "pageLoadTimeout"_s, &opt_in_pageLoadTimeout_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.waitForNavigationToComplete' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::WaitForNavigationToCompleteCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::WaitForNavigationToCompleteCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->waitForNavigationToComplete(in_browsingContextHandle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, opt_in_pageLoadStrategy_valueFound ? &opt_in_pageLoadStrategy : nullptr, opt_in_pageLoadTimeout_valueFound ? &opt_in_pageLoadTimeout : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::EvaluateJavaScriptFunctionCallback::EvaluateJavaScriptFunctionCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::EvaluateJavaScriptFunctionCallback::sendSuccess(const String& result)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setString("result"_s, result);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::evaluateJavaScriptFunction(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    String in_function = m_backendDispatcher->getString(parameters.get(), "function"_s, nullptr);
    RefPtr<JSON::Array> in_arguments = m_backendDispatcher->getArray(parameters.get(), "arguments"_s, nullptr);
    bool opt_in_expectsImplicitCallbackArgument_valueFound = false;
    bool opt_in_expectsImplicitCallbackArgument = m_backendDispatcher->getBoolean(parameters.get(), "expectsImplicitCallbackArgument"_s, &opt_in_expectsImplicitCallbackArgument_valueFound);
    bool opt_in_callbackTimeout_valueFound = false;
    int opt_in_callbackTimeout = m_backendDispatcher->getInteger(parameters.get(), "callbackTimeout"_s, &opt_in_callbackTimeout_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.evaluateJavaScriptFunction' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::EvaluateJavaScriptFunctionCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::EvaluateJavaScriptFunctionCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->evaluateJavaScriptFunction(in_browsingContextHandle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, in_function, *in_arguments, opt_in_expectsImplicitCallbackArgument_valueFound ? &opt_in_expectsImplicitCallbackArgument : nullptr, opt_in_callbackTimeout_valueFound ? &opt_in_callbackTimeout : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::PerformMouseInteractionCallback::PerformMouseInteractionCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::PerformMouseInteractionCallback::sendSuccess(RefPtr<Inspector::Protocol::Automation::Point>&& position)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setObject("position"_s, position);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::performMouseInteraction(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    RefPtr<JSON::Object> in_position = m_backendDispatcher->getObject(parameters.get(), "position"_s, nullptr);
    String in_button = m_backendDispatcher->getString(parameters.get(), "button"_s, nullptr);
    String in_interaction = m_backendDispatcher->getString(parameters.get(), "interaction"_s, nullptr);
    RefPtr<JSON::Array> in_modifiers = m_backendDispatcher->getArray(parameters.get(), "modifiers"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.performMouseInteraction' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::PerformMouseInteractionCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::PerformMouseInteractionCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->performMouseInteraction(in_handle, *in_position, in_button, in_interaction, *in_modifiers, callback.copyRef());

}

AutomationBackendDispatcherHandler::PerformKeyboardInteractionsCallback::PerformKeyboardInteractionsCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::PerformKeyboardInteractionsCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::performKeyboardInteractions(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    RefPtr<JSON::Array> in_interactions = m_backendDispatcher->getArray(parameters.get(), "interactions"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.performKeyboardInteractions' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::PerformKeyboardInteractionsCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::PerformKeyboardInteractionsCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->performKeyboardInteractions(in_handle, *in_interactions, callback.copyRef());

}

AutomationBackendDispatcherHandler::PerformInteractionSequenceCallback::PerformInteractionSequenceCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::PerformInteractionSequenceCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::performInteractionSequence(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    RefPtr<JSON::Array> in_inputSources = m_backendDispatcher->getArray(parameters.get(), "inputSources"_s, nullptr);
    RefPtr<JSON::Array> in_steps = m_backendDispatcher->getArray(parameters.get(), "steps"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.performInteractionSequence' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::PerformInteractionSequenceCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::PerformInteractionSequenceCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->performInteractionSequence(in_handle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, *in_inputSources, *in_steps, callback.copyRef());

}

AutomationBackendDispatcherHandler::CancelInteractionSequenceCallback::CancelInteractionSequenceCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::CancelInteractionSequenceCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::cancelInteractionSequence(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.cancelInteractionSequence' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::CancelInteractionSequenceCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::CancelInteractionSequenceCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->cancelInteractionSequence(in_handle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::TakeScreenshotCallback::TakeScreenshotCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::TakeScreenshotCallback::sendSuccess(const String& data)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setString("data"_s, data);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::takeScreenshot(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_handle = m_backendDispatcher->getString(parameters.get(), "handle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    bool opt_in_nodeHandle_valueFound = false;
    String opt_in_nodeHandle = m_backendDispatcher->getString(parameters.get(), "nodeHandle"_s, &opt_in_nodeHandle_valueFound);
    bool opt_in_scrollIntoViewIfNeeded_valueFound = false;
    bool opt_in_scrollIntoViewIfNeeded = m_backendDispatcher->getBoolean(parameters.get(), "scrollIntoViewIfNeeded"_s, &opt_in_scrollIntoViewIfNeeded_valueFound);
    bool opt_in_clipToViewport_valueFound = false;
    bool opt_in_clipToViewport = m_backendDispatcher->getBoolean(parameters.get(), "clipToViewport"_s, &opt_in_clipToViewport_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.takeScreenshot' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::TakeScreenshotCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::TakeScreenshotCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->takeScreenshot(in_handle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, opt_in_nodeHandle_valueFound ? &opt_in_nodeHandle : nullptr, opt_in_scrollIntoViewIfNeeded_valueFound ? &opt_in_scrollIntoViewIfNeeded : nullptr, opt_in_clipToViewport_valueFound ? &opt_in_clipToViewport : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::ResolveChildFrameHandleCallback::ResolveChildFrameHandleCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::ResolveChildFrameHandleCallback::sendSuccess(const String& result)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setString("result"_s, result);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::resolveChildFrameHandle(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    bool opt_in_frameHandle_valueFound = false;
    String opt_in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, &opt_in_frameHandle_valueFound);
    bool opt_in_ordinal_valueFound = false;
    int opt_in_ordinal = m_backendDispatcher->getInteger(parameters.get(), "ordinal"_s, &opt_in_ordinal_valueFound);
    bool opt_in_name_valueFound = false;
    String opt_in_name = m_backendDispatcher->getString(parameters.get(), "name"_s, &opt_in_name_valueFound);
    bool opt_in_nodeHandle_valueFound = false;
    String opt_in_nodeHandle = m_backendDispatcher->getString(parameters.get(), "nodeHandle"_s, &opt_in_nodeHandle_valueFound);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.resolveChildFrameHandle' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::ResolveChildFrameHandleCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::ResolveChildFrameHandleCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->resolveChildFrameHandle(in_browsingContextHandle, opt_in_frameHandle_valueFound ? &opt_in_frameHandle : nullptr, opt_in_ordinal_valueFound ? &opt_in_ordinal : nullptr, opt_in_name_valueFound ? &opt_in_name : nullptr, opt_in_nodeHandle_valueFound ? &opt_in_nodeHandle : nullptr, callback.copyRef());

}

AutomationBackendDispatcherHandler::ResolveParentFrameHandleCallback::ResolveParentFrameHandleCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::ResolveParentFrameHandleCallback::sendSuccess(const String& result)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setString("result"_s, result);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::resolveParentFrameHandle(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    String in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.resolveParentFrameHandle' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::ResolveParentFrameHandleCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::ResolveParentFrameHandleCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->resolveParentFrameHandle(in_browsingContextHandle, in_frameHandle, callback.copyRef());

}

AutomationBackendDispatcherHandler::ComputeElementLayoutCallback::ComputeElementLayoutCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::ComputeElementLayoutCallback::sendSuccess(RefPtr<Inspector::Protocol::Automation::Rect>&& rect, RefPtr<Inspector::Protocol::Automation::Point>&& inViewCenterPoint, bool isObscured)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setObject("rect"_s, rect);
    if (inViewCenterPoint)
        jsonMessage->setObject("inViewCenterPoint"_s, inViewCenterPoint);
    jsonMessage->setBoolean("isObscured"_s, isObscured);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::computeElementLayout(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    String in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, nullptr);
    String in_nodeHandle = m_backendDispatcher->getString(parameters.get(), "nodeHandle"_s, nullptr);
    bool opt_in_scrollIntoViewIfNeeded_valueFound = false;
    bool opt_in_scrollIntoViewIfNeeded = m_backendDispatcher->getBoolean(parameters.get(), "scrollIntoViewIfNeeded"_s, &opt_in_scrollIntoViewIfNeeded_valueFound);
    String in_coordinateSystem = m_backendDispatcher->getString(parameters.get(), "coordinateSystem"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.computeElementLayout' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::ComputeElementLayoutCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::ComputeElementLayoutCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->computeElementLayout(in_browsingContextHandle, in_frameHandle, in_nodeHandle, opt_in_scrollIntoViewIfNeeded_valueFound ? &opt_in_scrollIntoViewIfNeeded : nullptr, in_coordinateSystem, callback.copyRef());

}

AutomationBackendDispatcherHandler::SelectOptionElementCallback::SelectOptionElementCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::SelectOptionElementCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::selectOptionElement(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    String in_frameHandle = m_backendDispatcher->getString(parameters.get(), "frameHandle"_s, nullptr);
    String in_nodeHandle = m_backendDispatcher->getString(parameters.get(), "nodeHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.selectOptionElement' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::SelectOptionElementCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::SelectOptionElementCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->selectOptionElement(in_browsingContextHandle, in_frameHandle, in_nodeHandle, callback.copyRef());

}

void AutomationBackendDispatcher::isShowingJavaScriptDialog(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.isShowingJavaScriptDialog' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    bool out_result;
    m_agent->isShowingJavaScriptDialog(error, in_browsingContextHandle, &out_result);

    if (!error.length())
        result->setBoolean("result"_s, out_result);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::dismissCurrentJavaScriptDialog(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.dismissCurrentJavaScriptDialog' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->dismissCurrentJavaScriptDialog(error, in_browsingContextHandle);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::acceptCurrentJavaScriptDialog(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.acceptCurrentJavaScriptDialog' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->acceptCurrentJavaScriptDialog(error, in_browsingContextHandle);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::messageOfCurrentJavaScriptDialog(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.messageOfCurrentJavaScriptDialog' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    String out_message;
    m_agent->messageOfCurrentJavaScriptDialog(error, in_browsingContextHandle, &out_message);

    if (!error.length())
        result->setString("message"_s, out_message);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::setUserInputForCurrentJavaScriptPrompt(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    String in_userInput = m_backendDispatcher->getString(parameters.get(), "userInput"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.setUserInputForCurrentJavaScriptPrompt' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->setUserInputForCurrentJavaScriptPrompt(error, in_browsingContextHandle, in_userInput);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::setFilesToSelectForFileUpload(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    RefPtr<JSON::Array> in_filenames = m_backendDispatcher->getArray(parameters.get(), "filenames"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.setFilesToSelectForFileUpload' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->setFilesToSelectForFileUpload(error, in_browsingContextHandle, *in_filenames);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

AutomationBackendDispatcherHandler::GetAllCookiesCallback::GetAllCookiesCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::GetAllCookiesCallback::sendSuccess(RefPtr<JSON::ArrayOf<Inspector::Protocol::Automation::Cookie>>&& cookies)
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();
    jsonMessage->setArray("cookies"_s, cookies);
    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::getAllCookies(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.getAllCookies' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::GetAllCookiesCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::GetAllCookiesCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->getAllCookies(in_browsingContextHandle, callback.copyRef());

}

AutomationBackendDispatcherHandler::DeleteSingleCookieCallback::DeleteSingleCookieCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::DeleteSingleCookieCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::deleteSingleCookie(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    String in_cookieName = m_backendDispatcher->getString(parameters.get(), "cookieName"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.deleteSingleCookie' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::DeleteSingleCookieCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::DeleteSingleCookieCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->deleteSingleCookie(in_browsingContextHandle, in_cookieName, callback.copyRef());

}

AutomationBackendDispatcherHandler::AddSingleCookieCallback::AddSingleCookieCallback(Ref<BackendDispatcher>&& backendDispatcher, int id) : BackendDispatcher::CallbackBase(WTFMove(backendDispatcher), id) { }

void AutomationBackendDispatcherHandler::AddSingleCookieCallback::sendSuccess()
{
    Ref<JSON::Object> jsonMessage = JSON::Object::create();

    CallbackBase::sendSuccess(WTFMove(jsonMessage));
}

void AutomationBackendDispatcher::addSingleCookie(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    RefPtr<JSON::Object> in_cookie = m_backendDispatcher->getObject(parameters.get(), "cookie"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.addSingleCookie' can't be processed"_s);
        return;
    }

    Ref<AutomationBackendDispatcherHandler::AddSingleCookieCallback> callback = adoptRef(*new AutomationBackendDispatcherHandler::AddSingleCookieCallback(m_backendDispatcher.copyRef(), requestId));
    m_agent->addSingleCookie(in_browsingContextHandle, *in_cookie, callback.copyRef());

}

void AutomationBackendDispatcher::deleteAllCookies(long requestId, RefPtr<JSON::Object>&& parameters)
{
    String in_browsingContextHandle = m_backendDispatcher->getString(parameters.get(), "browsingContextHandle"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.deleteAllCookies' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->deleteAllCookies(error, in_browsingContextHandle);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::getSessionPermissions(long requestId, RefPtr<JSON::Object>&&)
{
    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    RefPtr<JSON::ArrayOf<Inspector::Protocol::Automation::SessionPermissionData>> out_permissions;
    m_agent->getSessionPermissions(error, out_permissions);

    if (!error.length())
        result->setArray("permissions"_s, out_permissions);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

void AutomationBackendDispatcher::setSessionPermissions(long requestId, RefPtr<JSON::Object>&& parameters)
{
    RefPtr<JSON::Array> in_permissions = m_backendDispatcher->getArray(parameters.get(), "permissions"_s, nullptr);
    if (m_backendDispatcher->hasProtocolErrors()) {
        m_backendDispatcher->reportProtocolError(BackendDispatcher::InvalidParams, "Some arguments of method 'Automation.setSessionPermissions' can't be processed"_s);
        return;
    }

    ErrorString error;
    Ref<JSON::Object> result = JSON::Object::create();
    m_agent->setSessionPermissions(error, *in_permissions);

    if (!error.length())
        m_backendDispatcher->sendResponse(requestId, WTFMove(result), false);
    else
        m_backendDispatcher->reportProtocolError(BackendDispatcher::ServerError, WTFMove(error));
}

} // namespace Inspector
