/*
 * Copyright (C) 2010-2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "CacheStorageEngineConnection.h"

#include "ArgumentCoders.h"
#include "CacheStorageEngineConnectionMessages.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/CacheQueryOptions.h>
#include <WebCore/ClientOrigin.h>
#include <WebCore/DOMCacheEngine.h>
#include <WebCore/ResourceRequest.h>
#include <pal/SessionID.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebKit {

void CacheStorageEngineConnection::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::Reference::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::Reference>(decoder, this, &CacheStorageEngineConnection::reference);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::Dereference::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::Dereference>(decoder, this, &CacheStorageEngineConnection::dereference);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::Open::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::Open>(decoder, this, &CacheStorageEngineConnection::open);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::Remove::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::Remove>(decoder, this, &CacheStorageEngineConnection::remove);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::Caches::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::Caches>(decoder, this, &CacheStorageEngineConnection::caches);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::ClearMemoryRepresentation::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::ClearMemoryRepresentation>(decoder, this, &CacheStorageEngineConnection::clearMemoryRepresentation);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::EngineRepresentation::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::EngineRepresentation>(decoder, this, &CacheStorageEngineConnection::engineRepresentation);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::RetrieveRecords::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::RetrieveRecords>(decoder, this, &CacheStorageEngineConnection::retrieveRecords);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::DeleteMatchingRecords::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::DeleteMatchingRecords>(decoder, this, &CacheStorageEngineConnection::deleteMatchingRecords);
        return;
    }
    if (decoder.messageName() == Messages::CacheStorageEngineConnection::PutRecords::name()) {
        IPC::handleMessage<Messages::CacheStorageEngineConnection::PutRecords>(decoder, this, &CacheStorageEngineConnection::putRecords);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

