/*
 * Copyright (C) 2010-2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "WebCacheStorageConnection.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "WebCacheStorageConnectionMessages.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/DOMCacheEngine.h>
#include <wtf/Optional.h>
#include <wtf/text/WTFString.h>

namespace WebKit {

void WebCacheStorageConnection::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::WebCacheStorageConnection::OpenCompleted::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::OpenCompleted>(decoder, this, &WebCacheStorageConnection::openCompleted);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::RemoveCompleted::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::RemoveCompleted>(decoder, this, &WebCacheStorageConnection::removeCompleted);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::UpdateCaches::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::UpdateCaches>(decoder, this, &WebCacheStorageConnection::updateCaches);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::UpdateRecords::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::UpdateRecords>(decoder, this, &WebCacheStorageConnection::updateRecords);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::DeleteRecordsCompleted::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::DeleteRecordsCompleted>(decoder, this, &WebCacheStorageConnection::deleteRecordsCompleted);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::PutRecordsCompleted::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::PutRecordsCompleted>(decoder, this, &WebCacheStorageConnection::putRecordsCompleted);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::EngineRepresentationCompleted::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::EngineRepresentationCompleted>(decoder, this, &WebCacheStorageConnection::engineRepresentationCompleted);
        return;
    }
    if (decoder.messageName() == Messages::WebCacheStorageConnection::ClearMemoryRepresentationCompleted::name()) {
        IPC::handleMessage<Messages::WebCacheStorageConnection::ClearMemoryRepresentationCompleted>(decoder, this, &WebCacheStorageConnection::clearMemoryRepresentationCompleted);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

