/*
 * Copyright (C) 2010-2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "ArgumentCoders.h"
#include <WebCore/DOMCacheEngine.h>
#include <wtf/Forward.h>
#include <wtf/Optional.h>
#include <wtf/text/WTFString.h>


namespace Messages {
namespace WebCacheStorageConnection {

static inline IPC::StringReference messageReceiverName()
{
    return IPC::StringReference("WebCacheStorageConnection");
}

class OpenCompleted {
public:
    typedef std::tuple<uint64_t, const WebCore::DOMCacheEngine::CacheIdentifierOrError&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("OpenCompleted"); }
    static const bool isSync = false;

    OpenCompleted(uint64_t requestIdentifier, const WebCore::DOMCacheEngine::CacheIdentifierOrError& result)
        : m_arguments(requestIdentifier, result)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RemoveCompleted {
public:
    typedef std::tuple<uint64_t, const WebCore::DOMCacheEngine::CacheIdentifierOrError&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("RemoveCompleted"); }
    static const bool isSync = false;

    RemoveCompleted(uint64_t requestIdentifier, const WebCore::DOMCacheEngine::CacheIdentifierOrError& result)
        : m_arguments(requestIdentifier, result)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UpdateCaches {
public:
    typedef std::tuple<uint64_t, const WebCore::DOMCacheEngine::CacheInfosOrError&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("UpdateCaches"); }
    static const bool isSync = false;

    UpdateCaches(uint64_t requestIdentifier, const WebCore::DOMCacheEngine::CacheInfosOrError& result)
        : m_arguments(requestIdentifier, result)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UpdateRecords {
public:
    typedef std::tuple<uint64_t, const WebCore::DOMCacheEngine::RecordsOrError&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("UpdateRecords"); }
    static const bool isSync = false;

    UpdateRecords(uint64_t requestIdentifier, const WebCore::DOMCacheEngine::RecordsOrError& result)
        : m_arguments(requestIdentifier, result)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DeleteRecordsCompleted {
public:
    typedef std::tuple<uint64_t, const WebCore::DOMCacheEngine::RecordIdentifiersOrError&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("DeleteRecordsCompleted"); }
    static const bool isSync = false;

    DeleteRecordsCompleted(uint64_t requestIdentifier, const WebCore::DOMCacheEngine::RecordIdentifiersOrError& result)
        : m_arguments(requestIdentifier, result)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PutRecordsCompleted {
public:
    typedef std::tuple<uint64_t, const WebCore::DOMCacheEngine::RecordIdentifiersOrError&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("PutRecordsCompleted"); }
    static const bool isSync = false;

    PutRecordsCompleted(uint64_t requestIdentifier, const WebCore::DOMCacheEngine::RecordIdentifiersOrError& result)
        : m_arguments(requestIdentifier, result)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class EngineRepresentationCompleted {
public:
    typedef std::tuple<uint64_t, const String&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("EngineRepresentationCompleted"); }
    static const bool isSync = false;

    EngineRepresentationCompleted(uint64_t requestIdentifier, const String& representation)
        : m_arguments(requestIdentifier, representation)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ClearMemoryRepresentationCompleted {
public:
    typedef std::tuple<uint64_t, const Optional<WebCore::DOMCacheEngine::Error>&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("ClearMemoryRepresentationCompleted"); }
    static const bool isSync = false;

    ClearMemoryRepresentationCompleted(uint64_t requestIdentifier, const Optional<WebCore::DOMCacheEngine::Error>& error)
        : m_arguments(requestIdentifier, error)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

} // namespace WebCacheStorageConnection
} // namespace Messages
