/*
 * Copyright (C) 2010-2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#if ENABLE(NETSCAPE_PLUGIN_API)

#include "WebProcessConnection.h"

#include "Decoder.h"
#include "HandleMessage.h"
#include "PluginCreationParameters.h"
#include "WebProcessConnectionMessages.h"

namespace Messages {

namespace WebProcessConnection {

void CreatePlugin::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, bool creationResult, bool wantsWheelEvents, uint32_t remoteLayerClientID)
{
    *encoder << creationResult;
    *encoder << wantsWheelEvents;
    *encoder << remoteLayerClientID;
    connection.sendSyncReply(WTFMove(encoder));
}

void DestroyPlugin::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace WebProcessConnection

} // namespace Messages

namespace WebKit {

void WebProcessConnection::didReceiveWebProcessConnectionMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::WebProcessConnection::CreatePluginAsynchronously::name()) {
        IPC::handleMessage<Messages::WebProcessConnection::CreatePluginAsynchronously>(decoder, this, &WebProcessConnection::createPluginAsynchronously);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

void WebProcessConnection::didReceiveSyncWebProcessConnectionMessage(IPC::Connection& connection, IPC::Decoder& decoder, std::unique_ptr<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::WebProcessConnection::CreatePlugin::name()) {
        IPC::handleMessageDelayed<Messages::WebProcessConnection::CreatePlugin>(connection, decoder, replyEncoder, this, &WebProcessConnection::createPlugin);
        return;
    }
    if (decoder.messageName() == Messages::WebProcessConnection::DestroyPlugin::name()) {
        IPC::handleMessageDelayed<Messages::WebProcessConnection::DestroyPlugin>(connection, decoder, replyEncoder, this, &WebProcessConnection::destroyPlugin);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit


#endif // ENABLE(NETSCAPE_PLUGIN_API)
